
/**
 * Attach editors.
 */
Drupal.editorAttach = function() {   
  Drupal.editors = [];
  
  var selector = ''; 
  var visibility = Drupal.settings.visibility_api.editor;
  
  if (!visibility.visible){
    return;
  }
  
  if (visibility.selector_visibility == 'include'){ 
    // Compile selectors
    $.each(visibility.selectors, function(i, e){
        selector += 'textarea' + e + ',';
    });
    
    // Editors
    $(selector).each(function(){
      Drupal.editors.push(new Drupal.editor(this));  
    });
  }
  else if (visibility.selector_visibility == 'exclude'){      
    // Compile selectors
    $.each(visibility.selectors, function(i, e){
        selector +=  e + ',';
    });
                  
    // Editors
    // @todo: fix / finish and clean up
    $('textarea').not(selector).each(function(){ 
      Drupal.editors.push(new Drupal.editor(this));  
    });    
  }
  else {
    return;
  }

  if (Drupal.editors.length){
    // When a form is submitted we need to make sure each editor
    // textarea has the finalized output, ensuring that changes are retained
    $('form').submit(function(){
      $.each(Drupal.editors, function(i, e){    
        e.populateTextarea();
      });
    });
  }
};                              

/**
 * Editor object.
 *
 * Properties:
 *   - container
 *   - currentMode
 *   - plugins
 *   - preview  
 *   - previewWindow
 *   - previewDocument
 *   - settings 
 *   - options
 *   - optionsEffect
 *   - textarea
 *   - toolbar
 *
 * Methods:
 *   - togglePreview
 *   - populatePreview
 *   - populateTextarea
 *   - showOptions
 *   - hideOptions
 *   - getSelection
 *   - getSelectedText
 *   - getSelectedRange
 *   - getSelectedParent
 *   - getPluginPID
 *   - execCommand
 *   - selectAll
 *   - undo
 *   - redo
 *   - copy
 *   - paste
 *   - remove
 *   - removeFormat
 *   - insert
 *   - setMessage
 *
 * For additional execCommand commands and reference visit: 
 *   - Mozilla: http://www.mozilla.org/editor/midas-spec.html
 *   - Microsoft: http://msdn2.microsoft.com/en-us/library/aa741314%28VS.85%29.aspx
 *
 * @todo: revise doc above
 */
Drupal.editor = function(textarea) {
  var ed = this; 
                      
  // Container
  $(textarea).wrap('<div class="editor-container"></div>');
  this.container = $(textarea).parent('.editor-container'); 
  
  // Prepare toolbars, options, and preview    
  $('.grippie', this.container).hide();
  $(textarea).hide();
  $(textarea).before(Drupal.settings.editor.toolbars);
  $(textarea).before('<div class="editor-options"></div>');
  $(textarea).before('<iframe class="editor-preview" src="javascript:;" style="width: 100%; height: 200px;"></iframe>');
     
  // Default properties
  this.textarea = textarea;
  this.preview = $(this.container).children('.editor-preview');
  this.toolbars = $(this.container).children('.editor-toolbar');
  this.options = $(this.container).children('.editor-options');  
  this.plugins = $('.editor-plugin', this.toolbars); 
  this.settings = Drupal.settings.editor;
  this.currentMode = 'preview';
  this.previewWindow = this.preview[0].contentWindow;
  this.previewDocument = $('.editor-preview').contentDocument();
  this.optionsEffect = 'fade';
  
  // Preview events
  $(this.preview).mousedown(function(e){
    $(ed).trigger('mouseDown', [e]);
  })
                      
  // Plugin events
  $(this.plugins).hover(function(){
    $(this).addClass('hover');
  }, function(){
    $(this).removeClass('hover');    
  })
  .mousedown(function(){  
    var PID = ed.getPluginPID(this);       
    $(this).addClass('active');  
    ed.hideOptions();
	
	// Dispatch PIDMouseDown event
  // @todo: pass this plugin as arg
   $(ed).trigger(PID + 'MouseDown');
  })
  .mouseup(function(){
    $(this).removeClass('active');
  });    
  
  // Design mode
  $(this.preview).designMode('on'); 
       
  // Display preview 
  this.populatePreview(); 
  
  // Hide options 
  $(this.options).hide();
  
  // Invoke plugin init handlers
  $.each(Drupal.settings.editor.plugins, function(i, e){
    eval('ed.' + e.pid + 'Init(this);');
  });
};

/**
 * Write to the preview iframe.
 */
Drupal.editor.prototype.previewWrite = function(markup) {
  this.previewDocument.open();                 
  this.previewDocument.write(markup);                 
  this.previewDocument.close(); 
};

/**
 * Toggle display of the WYSIWYG preview.
 *
 * @todo: clean up and refactor  
 */
Drupal.editor.prototype.togglePreview = function() {
  var ed = this;
  var textareaDisplay = $(this.textarea).css('display');
  
  if (textareaDisplay == 'none'){
    // Display textarea
    $(this.toolbars).hide();
    $(this.options).hide();
    $(this.plugins).hide();
    $(this.preview).hide();
    $(this.textarea).show();
    $('.grippie', this.container).show();
    $(this.textarea).before('<a href="#" class="editor-start-preview">Preview</a>');
    this.populateTextarea();
    this.currentMode = 'textarea';
    
    // Listen to editor start preview click
    $('.editor-start-preview', this.container).click(function(){
      ed.togglePreview();  
    });
	
	  // Dispatch preview event
	  $(ed).trigger('preview', ['hidden']);
  }                                      
  else {
    // Display preview
    $(this.toolbars).show();
    $(this.plugins).show();
    $(this.preview).show();
    $(this.textarea).hide();
    $('.editor-start-preview, .grippie', this.container).hide();
    this.populatePreview();
    this.currentMode = 'preview'; 
	
	  // Dispatch preview event
	  $(ed).trigger('preview', ['visible']);
  }
};
                   
/**
 * Populate the editor preview based on the textarea content.
 */                          
Drupal.editor.prototype.populatePreview = function() {
  this.previewWrite($(this.textarea).val());
};

/**
 * Populate the editor textarea with HTML based on the previews content.
 * 
 * @todo fix
 */                          
Drupal.editor.prototype.populateTextarea = function() {  
  $(this.textarea).val(this.previewDocument.body.innerHTML);
};

/**
 * Display options panel.
 */
Drupal.editor.prototype.showOptions = function(options) {  
  // Option effect
  switch(this.optionsEffect){
    case 'slide':
      $(this.options).slideDown(300).html(options);  
      break;
    
    default:
      $(this.options).fadeIn(300).html(options);  
  }
  
  // Clear click-clear fields when clicked, 
  // restoring initial values when nothing is present
  $('.click-clear').click(function(){
    var val = $(this).val();
    $(this).val('');
    
    $(this).blur(function(){
      if ($(this).val() == ''){
        $(this).val(val);
      }
    });
  });
};

/**
 * Remove options panel.
 */
Drupal.editor.prototype.hideOptions = function() {
  var display = $(this.options).css('display'); 
  
  if (display != 'none'){
    // Option effect
    switch(this.optionsEffect){
      case 'slide':
        $(this.options).slideUp(300);  
        break;
      
      default:
        $(this.options).fadeOut(300);  
    }
  }
};

/**
 * Execute a designMode command.
 *
 * @param string command
 *   Command to execute.
 *
 * @param string value
 *   (optiona) Not all commands require a value.
 *
 * @return object
 *   Editor
 *
 * For additional execCommand commands and reference visit: 
 *   - Mozilla: http://www.mozilla.org/editor/midas-spec.html
 *   - Microsoft: http://msdn2.microsoft.com/en-us/library/aa741314%28VS.85%29.aspx
 */
Drupal.editor.prototype.execCommand = function(command, value) {
  $(this.preview).execCommand(command, value); 
  return this;
};

/**
 * Undo.
 * 
 * @return object
 *   Editor
 */
Drupal.editor.prototype.undo = function() {
  return this.execCommand('undo'); 
};

/**
 * Redo. 
 * 
 * @return object
 *   Editor
 */
Drupal.editor.prototype.redo = function() {
  return this.execCommand('redo');
};

/**
 * Copy.
 *
 * @return object
 *   Editor
 */
Drupal.editor.prototype.copy = function() {
  return this.execCommand('copy'); 
};

/**
 * Paste.
 *
 * @return object
 *   Editor
 */
Drupal.editor.prototype.paste = function() {
  return this.execCommand('paste');  
};

/**
 * Select all.
 *
 * @return object
 *   Editor
 */
Drupal.editor.prototype.selectAll = function() {
  return this.execCommand('selectall');  
};

/**
 * Remove selected objects and text.
 *
 * @return object
 *   Editor
 */
Drupal.editor.prototype.remove = function() {
  return this.execCommand('delete');  
};

/**
 * Insert markup at the caret or current selection.
 * 
 * @param string markup
 * 
 * @return object
 * 	Editor
 */
Drupal.editor.prototype.insert = function(markup) {
  return this.execCommand('inserthtml', markup);  
};

/**
 * Wrap current selection.
 *
 * @todo: fix textwrapping and test this alot
 */
Drupal.editor.prototype.wrap = function(prefix, suffix) {
    
  // Extract text fragment which is then wrapped in our prefix/suffix
  var selection = this.getSelectionRange().extractContents();
  selection.textContent = prefix + selection.textContent + suffix;  
  
  // Place new content at the proper offset within the old text fragment 
  var anchorNode = this.getSelection().anchorNode;
  var offset = this.getSelection().anchorOffset;
  var start = anchorNode.textContent.slice(0, offset);  
  var end = anchorNode.textContent.slice(offset, anchorNode.textContent.length);  
  anchorNode.textContent = start + selection.textContent + end;
          
  return this; 
};  

/**
 * Wrap selection with a node.
 *
 * @param object node
 *    A node object. To create a new node use the createNode() method.
 */       
Drupal.editor.prototype.wrapNode = function(node) {
  this.getSelectionRange().surroundContents(node);
  this.previewWindow.focus();  
};          

/**
 * Create a new node.
 *
 * @param string type
 *   Type of node to create, such as 
 *   p, em, strong, a, h1, h2, h3, h3 etc
 *
 * @return object
 */
Drupal.editor.prototype.createNode = function(type) {
  return document.createElement(type);
};

/**
 * Remove selected formatting.
 *
 * @return object
 *   Editor
 */
Drupal.editor.prototype.removeFormat = function() {
  return this.execCommand('removeformat');  
};

/**
 * Get selection object.
 *
 * @return object
 *   Selection
 */
Drupal.editor.prototype.getSelection = function() { 
  return this.previewWindow.getSelection();   
};

/**
 * Get selection range object.
 *
 * @return object
 *   Range
 */
Drupal.editor.prototype.getSelectionRange = function() {
  return this.getSelection().getRangeAt(0);
};

/**
 * Display a message.
 */
Drupal.editor.prototype.setMessage = function(message, type) {
  var ed = this;
  var type = type || 'status';
  
  $('.messages', this.container).remove();
  $(this.container).prepend('<div class="messages ' + type + '">' + message + '</div>');
  setTimeout('hideMessages()', 3000);
};

var hideMessages = function(){
  $('.messages').fadeOut(800);
};

/**
 * Return the plugin's PID.
 */
Drupal.editor.prototype.getPluginPID = function(plugin) {
  var id = $(plugin).attr('id');
  return id.replace('editor-', '');
};

if (Drupal.jsEnabled) {
  $(function(){
    Drupal.editorAttach();
  });
}