
/**
 * Undo plugin.
 */
Drupal.editor.prototype.undoInit = function(plugin) { 
  $(this).bind('undoMouseDown', function(e){ 
    this.undo();
  });
};

/**         
 * Redo plugin.
 */
Drupal.editor.prototype.redoInit = function(plugin) {
  $(this).bind('redoMouseDown', function(e){
     this.redo();
  });
};

/**
 * Italic plugin. Wrap selected text in <em> tags.
 */
Drupal.editor.prototype.italicInit = function(plugin) {
  $(this).bind('italicMouseDown', function(e){
    this.execCommand('italic');
  });                   
};

/**
 * Bold plugin. Wrap selected text in <strong> tags.
 */
Drupal.editor.prototype.boldInit = function(plugin) {
  $(this).bind('boldMouseDown', function(e){
    this.execCommand('bold'); 
  });
};

/**
 * Align Left plugin. Wrap selected text in <p style="text-align: left;"></p>.
 */
Drupal.editor.prototype.align_leftInit = function(plugin) {
  $(this).bind('align_leftMouseDown', function(e){	
    this.execCommand('justifyleft'); 
  });
};

/**
 * Align Right plugin. Wrap selected text in <p style="text-align: left;"></p>.
 */
Drupal.editor.prototype.align_rightInit = function(plugin) {
  $(this).bind('align_rightMouseDown', function(e){	
    this.execCommand('justifyright');
  });
};

/**
 * Align Center plugin. Wrap selected text in <p style="text-align: left;"></p>.
 */
Drupal.editor.prototype.align_centerInit = function(plugin) {
  $(this).bind('align_centerMouseDown', function(e){
    this.execCommand('justifycenter');
  });
};

/**
 * Align Justify plugin. Wrap selected text in <p style="text-align: justify;"></p>.
 */
Drupal.editor.prototype.align_justifyInit = function(plugin) {
  $(this).bind('align_justifyMouseDown', function(e){
    this.execCommand('justifyfull');
  });
};

/**
 * HTML plugin. Toggle preview.
 */
Drupal.editor.prototype.htmlInit = function(plugin) {
  $(this).bind('htmlMouseDown', function(e){	
    this.togglePreview();
  });
};

/**
 * Numeric List plugin. 
 */
Drupal.editor.prototype.list_numericInit = function(plugin) {
  $(this).bind('list_numericMouseDown', function(e){  
    this.execCommand('insertorderedlist');
  });
};

/**
 * Bullet List plugin. 
 */
Drupal.editor.prototype.list_bulletInit = function(plugin) {
  $(this).bind('list_bulletMouseDown', function(e){  
    this.execCommand('insertunorderedlist');
  });
};

/**
 * Remove Formattin plugin. 
 */
Drupal.editor.prototype.remove_formatInit = function(plugin) {
  $(this).bind('remove_formatMouseDown', function(e){  
    this.removeFormat()
  });
};


/**
 * Link plugin. Provides options for customizing anchor tags.
 * 
 * @todo: issue with undo/redo
 * @todo: issue with filling out options and loosing selected text
 */
Drupal.editor.prototype.linkInit = function(plugin) {
  $(this).bind('linkMouseDown', function(e){	
	  var ed = e.target;
	  
	  this.showOptions(this.settings.plugins.link.options);
	    
	  // Apply anchor tag
	  $('.submit', this.options).click(function(){
	    var uri = $('.href', this.options).val();
	    var window = $('.window', this.options).val();
      
      // Validation
      if (uri == 'Location'){
        ed.setMessage('Enter a location.', 'error');
        return false;
      }
      
	    if (window == 'same'){
	      ed.execCommand('createlink', uri);
	    }
	    else {
	      // @todo: with execCommand  
	    }
      
	    ed.hideOptions();  
	  });
  });
};   

/**
 * Mailto plugin. Provides options for customizing mailto anchor tags.
 */
Drupal.editor.prototype.mailtoInit = function(plugin) {
  $(this).bind('mailtoMouseDown', function(e){	
    var ed = e.target;
  
    this.showOptions(this.settings.plugins.mailto.options);

    // Apply anchor tag
    $('.submit', this.options).click(function(){
      var email = $('.email', this.options).val();
      
      // Validation
      if (email == 'Email'){
        ed.setMessage('Enter an email.', 'error');
        return false;
      }
      
      ed.execCommand('createlink', 'mailto:' + email);  
    
      ed.hideOptions();  
    });
  });
};  

/**
 * Code plugin. Provides options to choose type of code.
 *
 * @todo: wrap php
 */
Drupal.editor.prototype.codeInit = function(plugin) {
  $(this).bind('codeMouseDown', function(e){	
    var ed = e.target;
  
    this.showOptions(this.settings.plugins.code.options);

    // Apply URI
    $('.submit', this.options).click(function(){
      var type = $('.type', this.options).val();
    
      switch(type){
        case 'php':
          ed.wrap('<?php', '?>');
          break;
         
        default:
          ed.execCommand('formatblock', 'code');
      }
    
    
      ed.hideOptions();  
    });
  });
};   

/**
 * Image plugin.
 */
Drupal.editor.prototype.imageInit = function(plugin) {
  $(this).bind('imageMouseDown', function(e){	
    var ed = e.target;
  
    this.showOptions(this.settings.plugins.image.options);

    // Apply URI
    $('.submit', this.options).click(function(){
      var uri = $('.uri', this.options).val();
      var alt = $('.alt', this.options).val();
      
      // Validation
      if (uri == 'Image Path'){
        ed.setMessage('Enter a valid path to an image. Example "http://yoursite.com/logo.jpg".', 'error');
        return false;
      }
      else if (alt == 'Description'){
        ed.setMessage('Enter a description.', 'error');
        return false;
      }
      
      ed.insert('<img src="' + uri + '" alt="' + alt + '" />')
    
      ed.hideOptions();  
    });
  });
};   



